/******************************************************************************
 *                                                                            *
 * (c) Copyright 1993, 1994, 1996, 1997, 1998, 1999 by Sun Microsystems, Inc.             *
 *                                                                            *
 ******************************************************************************
 *                                                                            *
 *    File: 	   dtfile_stubdnd.c                                           *
 *                                                                            *
 *    Description: DND gui callback routines                                  *
 *                                                                            *
 ******************************************************************************/

#pragma ident   "@(#)dtfile_dnd_stubs.c 5.15 99/03/04 SMI; TSOL 2.x"


/******************************************************************************
 *                                                                            *
 *  Includes                                                                  *
 *                                                                            *
 ******************************************************************************/

#include <sys/stat.h>
#include <string.h>
#include <tsol/secflgs.h>

#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>

#include <Xm/Xm.h>
#include <Xm/PushB.h>

#include <tsol/Xtsol.h>
#include <tsol/label.h>
#include <Dt/HourGlass.h>
#include <Dt/label_clipping.h>

#include "Encaps.h"
#include "FileMgr.h"
#include "dtfile_dnd.h"
#include "Help.h"
#include "dtfile_dnd.h"
#include "tsoldnd.h"


/******************************************************************************
 *                                                                            *
 *  Global variables and constants                                            *
 *                                                                            *
 ******************************************************************************/

#define TMPNAME2    "/usr/tmp/dtfileXXXXXX"
#define TSOL_HELP_VOL "SdtTrust"

/******************************************************************************
 *  Function    : load_data_view                                              *
 *                                                                            *
 *  Description : loads src text in appropriate mode in the scroll window     *
 *                                                                            *
 *  Input       : int - mode of display                                       *
 *                                                                            *
 *  Ouput       : None                                                        *
 ******************************************************************************/

void
load_data_view(int mode)
{
	int 			c;
	unsigned long 		addr,
				sel_size;
	Bool			file_err;
	FILE 		   	*tmpf,
				*tmpfr;
	char 		   	*tmpfile,
				*text,
				*text2,
				*hp, 
				*sp,
				*src_file,
				*cvtstr = "0123456789abcdef",
				buffer[64],
				linebuf[8 + 1 + 50 + 1 + 16 + 1 + 1];
	unsigned char  		*inp;
	XmString		src_string;
	secflgs_t 		attr;
	struct stat		statb;

	if (dnd_shell->src_frame->src_mldpath != NULL) {

		src_file = strdup(dnd_shell->src_frame->src_mldpath);

	} else {
		XtVaGetValues(dnd_shell->src_frame->src_file,
					  XmNlabelString,
					  &src_string,
					  NULL);
		XmStringGetLtoR(src_string, XmFONTLIST_DEFAULT_TAG, &src_file);
		XmStringFree(src_string);
	}

	if (!stat(src_file, &statb)) {
		file_err = FALSE;
	}
	if (file_err) {
		text = "";
		XmTextSetString(dnd_shell->data_frame->view_text, text);
		return;
	}

	switch(mode) {
		case TEXT: /* display as text */
			if (!(text = XtMalloc((unsigned)(statb.st_size+1)))) {
				return;
			}
			tmpf = fopen(src_file, "r");
			if (!fread(text, sizeof(char), statb.st_size + 1, tmpf)) {
				fprintf(stderr, "Warning: may not have read entire file!\n");
			}
			text[statb.st_size] = 0;
			XmTextSetString(dnd_shell->data_frame->view_text, text);
			/*
	 		 * set converted file into text window
	 		 */
			XtFree(text);
			fclose(tmpf);
		break;
	
		case HEX: /* display as hex */

			/*
			 * Conversion of long files can take a while, so turn
			 * on the hour glass
			 */
			_DtTurnOnHourGlass(dnd_shell->dnd_shell);

			tmpfile = strdup(TMPNAME2);
			tmpf = fdopen(mkstemp(tmpfile), "w");
			if (!(text = XtMalloc((unsigned)(statb.st_size+1)))) {
				_DtTurnOffHourGlass(dnd_shell->dnd_shell);
				return;
			}
			tmpfr = fopen(src_file, "r");
			if (!fread(text, sizeof(char), statb.st_size + 1, tmpfr)) {
				fprintf(stderr, "Warning: may not have read entire file!\n");
			}
			text[statb.st_size] = 0;
			fclose(tmpfr);
			sel_size = statb.st_size;
			inp = (unsigned char *) text;
			strcpy(linebuf, "aaaaaaaa nn nn nn nn  nn nn nn nn  nn nn nn nn");
			strcat(linebuf, "  nn nn nn nn ................\n");
			for (addr = 0; addr < sel_size; addr += 16) {
				linebuf[0] = cvtstr[(addr >> 28) & 0xf];
				linebuf[1] = cvtstr[(addr >> 24) & 0xf];
				linebuf[2] = cvtstr[(addr >> 20) & 0xf];
				linebuf[3] = cvtstr[(addr >> 16) & 0xf];
				linebuf[4] = cvtstr[(addr >> 12) & 0xf];
				linebuf[5] = cvtstr[(addr >> 8) & 0xf];
				linebuf[6] = cvtstr[(addr >> 4) & 0xf];
				linebuf[7] = cvtstr[addr & 0xf];
				hp = &linebuf[8+1];
				sp = &linebuf[8+1+50+1];
				for (c = 0; c < 16; c++) {
					if (addr+c >= sel_size) {
						do {
							*hp++ = ' ';
						} while (hp < &linebuf[8+1+50+1]);
						*sp = '\0';
						break;
					}
					*hp++ = cvtstr[*inp >> 4];
					*hp++ = cvtstr[*inp & 0xf];
					hp += !((c+1) % 4) && c != 15 ? 2 : 1;
					if (isprint(*inp)) {
						*sp++ = (char)*inp;
					}
					else {
						*sp++ = '.';
					}
					inp++;
				}	/* end for c <= 16 */
				(void)fputs(linebuf, tmpf);
			}	/* for addr <= sel_size */
			(void)fclose(tmpf);
			/*
	 		 * set converted file into text window
	 		 */
			XtFree(text);
			if (stat(tmpfile, &statb) == -1) {
				_DtTurnOffHourGlass(dnd_shell->dnd_shell);
				return;
			}
			if (!(text2 = XtMalloc((unsigned)(statb.st_size+1)))) {
				_DtTurnOffHourGlass(dnd_shell->dnd_shell);
				return;
			}
			tmpfr = fopen(tmpfile, "r");
			if (!fread(text2, sizeof(char), statb.st_size + 1, tmpfr)) {
				fprintf(stderr, "Warning: may not have read entire file!\n");
			}
			text2[statb.st_size] = 0;
			fclose(tmpfr);
			XmTextSetString(dnd_shell->data_frame->view_text, text2);
			XtFree(text2);
			(void)unlink(tmpfile);
			(void)free(tmpfile);
			_DtTurnOffHourGlass(dnd_shell->dnd_shell);
		break;
	
		case NONE: /* do not display */
			if (!(text = XtMalloc((unsigned)(3)))) {
				return;
			}
			strcpy(text, "  ");
			XmTextSetString(dnd_shell->data_frame->view_text, text);
			XtFree(text);
		break;
	}	/* end switch mode */
	XtFree(src_file);
}	/* load_data_view */



/******************************************************************************
 *  Function    : update_source_label                                         *
 *                                                                            *
 *  Description : update the label displayed for the source file              *
 *                                                                            *
 *  Input       : *bclabel_t   new value for the label                        *
 *                Dimension    new width for the label field                  *
 *                                                                            *
 *  Ouput       : None                                                        *
 ******************************************************************************/

void 
update_source_label(bclabel_t *new_label, Dimension label_w)
{
	Display		*disp;
	XmFontList	fontlist;
	int		user_view_flag;
	XmString	label_string = (XmString)NULL;

	DPRINTF(("update_source_label: width %d\n", label_w));

	disp = XtDisplay(dnd_shell->src_frame->src_lbl);

	XtVaGetValues(dnd_shell->src_frame->src_lbl,
	    XmNfontList,
	    &fontlist,
	    NULL);

	label_string = Xbiltos(disp, bcltoil(new_label),
	    label_w, fontlist, NULL);
	dnd_shell->src_frame->src_il = XmStringCopy(label_string);
	XmStringFree(label_string);

        if(strstr(tsoluser_ent->labelview, TSOL_USER_VIEW_EXTERNAL_KW))
		user_view_flag = VIEW_EXTERNAL;
        else
		user_view_flag = VIEW_INTERNAL;

	/*
	 * This should really do an |= BRACKETED, but that
	 * doesn't work because of a bug in Xbsltos
	 */
	if (hide_il && !hide_sl)
		user_view_flag = BRACKETED;

	if (!hide_il && !hide_sl) {
		label_string = Xbcltos(disp,
		    new_label, label_w, fontlist, user_view_flag);
	} else if (hide_il) {
		label_string = Xbsltos(disp,
		    bcltosl(new_label), label_w, fontlist, user_view_flag);
	} else if (hide_sl) {
		label_string = Xbiltos(disp,
		    bcltoil(new_label), label_w, fontlist, user_view_flag);
	} else {
		/* This shouldn't happen, but just in case */
		label_string = (XmString)(" ");
	}
	
	XtVaSetValues(dnd_shell->src_frame->src_lbl,
				  XmNlabelString,
				  label_string,
				  NULL);
	XmStringFree(label_string);
}

/******************************************************************************
 *  Function    : update_destination_label                                    *
 *                                                                            *
 *  Description : update the destination label                                *
 *                Also make sure the IL is valid.  If it is not:              *
 *                    - Display just the SL for the Destination label         *
 *                                                                            *
 *  Input       : *bilabel_t   new value for the IL                           *
 *                Dimension    new width for the label field                  *
 *                                                                            *
 *  Ouput       : None                                                        *
 ******************************************************************************/

void 
update_destination_label(bilabel_t *new_il, Dimension label_w)
{
	char		*warning;
	XmString	label_str;
	Dimension	width;
	int		user_view_flag;
	Display		*disp;
	XmFontList	fontlist;

	DPRINTF(("update_destination_label: width %d\n", label_w));

	disp = XtDisplay(dnd_shell->src_frame->src_lbl);

	XtVaGetValues(dnd_shell->src_frame->src_lbl,
	    XmNfontList,
	    &fontlist,
	    NULL);

        if(strstr(tsoluser_ent->labelview, TSOL_USER_VIEW_EXTERNAL_KW))
		user_view_flag = VIEW_EXTERNAL;
        else
		user_view_flag = VIEW_INTERNAL;

	/*
	 * This should really do an |= BRACKETED, but that
	 * doesn't work because of a bug in Xbsltos
	 */
	if (hide_il && !hide_sl)
		user_view_flag = BRACKETED;

	/*
	 * Update the destination label
	 */

	current_dst_cl = win_dst_cl;
	setcil(&current_dst_cl, new_il);

	current_dst_il = new_il;

	/*
	 * Make sure the IL is valid for the destination SL.
	 * Dest SL must dominate the source IL.
	 */

	if (bldominates(bcltosl(&win_dst_cl), biltolev(new_il)))
		il_is_valid = 1;
	else
		il_is_valid = 0;

	if (!hide_il && !hide_sl) {
		if (il_is_valid) 
			label_str = Xbcltos(disp, &current_dst_cl,
			    label_w, fontlist, user_view_flag);
		else
			label_str = Xbsltos(disp, bcltosl(&current_dst_cl),
			    label_w, fontlist, BRACKETED);
	} else if (hide_il)  {
		label_str = Xbsltos(disp, bcltosl(&current_dst_cl),
		    label_w, fontlist, user_view_flag);
	} else if (hide_sl) {
		label_str = Xbiltos(disp, bcltoil(&current_dst_cl),
		    label_w, fontlist, user_view_flag);
	} else {
		/* This shouldn't happen, but just in case */
		label_str = (XmString)(" ");
	}

	XtVaSetValues(dnd_shell->dst_frame->dst_lbl,
	    XmNlabelString,
	    label_str,
	    NULL);
	XmStringFree(label_str);
}

/******************************************************************************
 *  Function    : update_information_label                                    *
 *                                                                            *
 *  Description : update the IL value displayed in the Choose IL From frame   *
 *                Also make sure the IL is valid.  If it is not:              *
 *                    - Dim the IL value in the Choose IL From frame          *
 *                    - Display an error msg in the Choose IL From frame      *
 *                    - Set the apply button insensitive                      *
 *                                                                            *
 *  Input       : *bilabel_t   new value for the IL                           *
 *                Dimension    new width for the label field                  *
 *                                                                            *
 *  Ouput       : None                                                        *
 ******************************************************************************/

void 
update_information_label(bilabel_t *new_il, Dimension label_w)
{
	char		*warning;
	XmString	label_str;
	Dimension	width;
	int		user_view_flag;
	Display		*disp;
	XmFontList	fontlist;

	DPRINTF(("update_information_label: width %d\n", label_w));

	disp = XtDisplay(dnd_shell->info_frame->info_lbl);

	XtVaGetValues(dnd_shell->info_frame->info_lbl,
	    XmNfontList,
	    &fontlist,
	    NULL);

        if(strstr(tsoluser_ent->labelview, TSOL_USER_VIEW_EXTERNAL_KW))
		user_view_flag = VIEW_EXTERNAL;
        else
		user_view_flag = VIEW_INTERNAL;

	/*
	 * This should really do an |= BRACKETED, but that
	 * doesn't work because of a bug in Xbsltos
	 */
	if (hide_il && !hide_sl)
		user_view_flag = BRACKETED;

	/*
	 * Make sure the IL is valid for the destination SL.
	 * Dest SL must dominate the source IL.
	 */

	if (bldominates(bcltosl(&win_dst_cl), biltolev(new_il))) {
		il_is_valid = 1;
		warning = " ";
		if (user_is_authorized)
			XtSetSensitive(dnd_shell->apply_btn, True);
	} else {
		il_is_valid = 0;
		warning = GETMESSAGE(TSOLSET,
		    144, "Selected IL not valid for destination");
		XtSetSensitive(dnd_shell->apply_btn, False);
	}

	/*
	 * Update the IL in the Choose IL From frame
	 * (if ILs are being displayed)
	 */

	if (!hide_il) {

		label_str = Xbiltos(disp, new_il,
		    label_w, fontlist, user_view_flag);

		XtVaSetValues(dnd_shell->info_frame->info_lbl,
		    XmNlabelString,
		    label_str,
		    NULL);
		XmStringFree (label_str);

		label_str = XmStringCreateLocalized(warning);
		XtVaSetValues(dnd_shell->info_frame->il_warning,
		    XmNlabelString, label_str, NULL);
		XmStringFree (label_str);
		XtSetSensitive(dnd_shell->info_frame->info_lbl, il_is_valid);
	}

}

/******************************************************************************
 *  Function    : get_src_label                                               *
 *                                                                            *
 *  Description : src callback routine from label builder                     *
 *                                                                            *
 *  Input       : Widget    - caller widget                                   *
 *                XtPointer - client data                                     *
 *                XtPointer - call data                                       *
 *                                                                            *
 *  Ouput       : None                                                        *
 ******************************************************************************/

void 
get_src_label(Widget w, XtPointer client_data, XtPointer call_data)
{
	char		*str;
	XmString	label_str;
	bilabel_t       *new_il;
	Dimension	label_w;
	ModLabelData	*modLabel_data = (ModLabelData *)client_data;

	new_il = (bilabel_t *)tsol_lbuild_get(modLabel_data, LBUILD_WORK_IL);

	XtVaGetValues(dnd_shell->dst_frame->dst_lbl,
	    XmNwidth,
	    &label_w,
	    NULL);
	update_destination_label(new_il, label_w);
	XtVaGetValues(dnd_shell->info_frame->info_lbl,
	    XmNwidth,
	    &label_w,
	    NULL);
	update_information_label(new_il, label_w);
    XTSOLMakeUntrustedWindow(XtDisplay(G_dialogShell), XtWindow(G_dialogShell));
	tsol_lbuild_set(dnd_shell->src_builder, LBUILD_SHOW, FALSE, NULL);
}


/******************************************************************************
 *  Function    : apply_callback                                              *
 *                                                                            *
 *  Description : apply button callback routine                               *
 *                                                                            *
 *  Input       : Widget    - caller widget                                   *
 *                XtPointer - client data                                     *
 *                XtPointer - call data                                       *
 *                                                                            *
 *  Ouput       : None                                                        *
 ******************************************************************************/

void 
apply_callback(Widget w, XtPointer client_data, XtPointer xt_call_data)
{
	int 		rc;
	char 	   *newIL;
	XmString	IL_string;

	XmPushButtonCallbackStruct *call_data = 
		(XmPushButtonCallbackStruct *)xt_call_data;

	XtVaGetValues(dnd_shell->info_frame->info_lbl,
				  XmNlabelString,
				  &IL_string,
				  NULL);
	XmStringGetLtoR(IL_string, XmFONTLIST_DEFAULT_TAG, &newIL);
    XTSOLMakeUntrustedWindow(XtDisplay(G_dialogShell), XtWindow(G_dialogShell));
	XtDestroyWidget(G_dialogShell);

	if (hide_il == False) {
		PipeWriteString(G_filop_confirm_fd, newIL);
	}
	rc = PIPEMSG_PROCEED;
	write(G_filop_confirm_fd, &rc, sizeof(int));
	G_filop_confirm_fd = -1;
	DPRINTF(("apply_callback - IL %s\n", newIL ? newIL : "NULL"));
	XtFree(newIL);
	newIL = NULL;
	exit(0);
}


/******************************************************************************
 *  Function    : cancel_callback                                             *
 *                                                                            *
 *  Description : cancel button callback routine                              *
 *                                                                            *
 *  Input       : Widget    - caller widget                                   *
 *                XtPointer - client data                                     *
 *                XtPointer - call data                                       *
 *                                                                            *
 *  Ouput       : None                                                        *
 ******************************************************************************/

void 
cancel_callback(Widget w, XtPointer client_data, XtPointer xt_call_data)
{
	int 		rc;
	XmPushButtonCallbackStruct *call_data = 
		(XmPushButtonCallbackStruct *)xt_call_data;
    XTSOLMakeUntrustedWindow(XtDisplay(G_dialogShell), XtWindow(G_dialogShell));
	XtDestroyWidget(dnd_shell->dnd_shell);

	PipeWriteString(G_filop_confirm_fd, " ");
	rc = PIPEMSG_CANCEL;
	write(G_filop_confirm_fd, &rc, sizeof(int));
	G_filop_confirm_fd = -1;
	DPRINTF(("cancel_callback\n"));

	exit(0);
}


/******************************************************************************
 *  Function    : help_callback                                               *
 *                                                                            *
 *  Description : help button callback routine                                *
 *                                                                            *
 *  Input       : Widget    - caller widget                                   *
 *                XtPointer - client data                                     *
 *                XtPointer - call data                                       *
 *                                                                            *
 *  Ouput       : None                                                        *
 ******************************************************************************/

void 
help_callback(Widget w, XtPointer client_data, XtPointer xt_call_data)
{
	XmPushButtonCallbackStruct *call_data = 
		(XmPushButtonCallbackStruct *)xt_call_data;

	/* I18N issue */
	TsolHelpView(w, TSOL_HELP_VOL, HELP_DND_DIALOG_STR, "Trusted Solaris Drag and Drop Help Viewer");

}


/******************************************************************************
 *  Function    : text_callback                                               *
 *                                                                            *
 *  Description : text button callback routine                                *
 *                                                                            *
 *  Input       : Widget    - caller widget                                   *
 *                XtPointer - client data                                     *
 *                XtPointer - call data                                       *
 *                                                                            *
 *  Ouput       : None                                                        *
 ******************************************************************************/

void 
text_callback(Widget w, XtPointer client_data, XtPointer xt_call_data)
{
	XmPushButtonCallbackStruct *call_data = 
		(XmPushButtonCallbackStruct *)xt_call_data;
	load_data_view(TEXT);
}


/******************************************************************************
 *  Function    : hex_callback                                                *
 *                                                                            *
 *  Description : hex button callback routine                                 *
 *                                                                            *
 *  Input       : Widget    - caller widget                                   *
 *                XtPointer - client data                                     *
 *                XtPointer - call data                                       *
 *                                                                            *
 *  Ouput       : None                                                        *
 ******************************************************************************/

void 
hex_callback(Widget w, XtPointer client_data, XtPointer xt_call_data)
{
	XmPushButtonCallbackStruct *call_data = 
		(XmPushButtonCallbackStruct *)xt_call_data;
	load_data_view(HEX);
}


/******************************************************************************
 *  Function    : none_callback                                               *
 *                                                                            *
 *  Description : none button callback routine                                *
 *                                                                            *
 *  Input       : Widget    - caller widget                                   *
 *                XtPointer - client data                                     *
 *                XtPointer - call data                                       *
 *                                                                            *
 *  Ouput       : None                                                        *
 ******************************************************************************/

void 
none_callback(Widget w, XtPointer client_data, XtPointer xt_call_data)
{
	XmPushButtonCallbackStruct *call_data = 
		(XmPushButtonCallbackStruct *)xt_call_data;
	load_data_view(NONE);
}


/******************************************************************************
 *  Function    : srcil_callback                                              *
 *                                                                            *
 *  Description : srcil pulldown menu button callback routine                 *
 *                                                                            *
 *  Input       : Widget    - caller widget                                   *
 *                XtPointer - client data                                     *
 *                XtPointer - call data                                       *
 *                                                                            *
 *  Ouput       : None                                                        *
 ******************************************************************************/

void 
srcil_callback(Widget w, XtPointer client_data, XtPointer xt_call_data)
{
	Dimension	label_w;

	XmPushButtonCallbackStruct *call_data = 
	    (XmPushButtonCallbackStruct *)xt_call_data;

	XtVaGetValues(dnd_shell->dst_frame->dst_lbl,
	    XmNwidth,
	    &label_w,
	    NULL);
	update_destination_label(bcltoil(&file_src_cl), label_w);
	XtVaGetValues(dnd_shell->info_frame->info_lbl,
	    XmNwidth,
	    &label_w,
	    NULL);
	update_information_label(bcltoil(&file_src_cl), label_w);
}


/******************************************************************************
 *  Function    : dstil_callback                                              *
 *                                                                            *
 *  Description : dstil pulldown menu button callback routine                 *
 *                                                                            *
 *  Input       : Widget    - caller widget                                   *
 *                XtPointer - client data                                     *
 *                XtPointer - call data                                       *
 *                                                                            *
 *  Ouput       : None                                                        *
 ******************************************************************************/

void 
dstil_callback(Widget w, XtPointer client_data, XtPointer xt_call_data)
{
	Dimension	label_w;

	XmPushButtonCallbackStruct *call_data = 
		(XmPushButtonCallbackStruct *)xt_call_data;

	XtVaGetValues(dnd_shell->dst_frame->dst_lbl,
	    XmNwidth,
	    &label_w,
	    NULL);
	update_destination_label(bcltoil(&dst_dir_cl), label_w);
	XtVaGetValues(dnd_shell->info_frame->info_lbl,
	    XmNwidth,
	    &label_w,
	    NULL);
	update_information_label(bcltoil(&dst_dir_cl), label_w);
}


/******************************************************************************
 *  Function    : builderil_callback                                          *
 *                                                                            *
 *  Description : builder pulldown menu button callback routine               *
 *                                                                            *
 *  Input       : Widget    - caller widget                                   *
 *                XtPointer - client data                                     *
 *                XtPointer - call data                                       *
 *                                                                            *
 *  Ouput       : None                                                        *
 ******************************************************************************/

void 
builderil_callback(Widget w, XtPointer client_data, XtPointer xt_call_data)
{
	XmPushButtonCallbackStruct *call_data = 
		(XmPushButtonCallbackStruct *)xt_call_data;
	XTSOLMakeTrustedWindow(XtDisplay(G_dialogShell),
	    XtWindow(G_dialogShell));
	tsol_lbuild_set(dnd_shell->src_builder, LBUILD_SHOW, TRUE, NULL);
}

/******************************************************************************
 *  Function    : resize_callback                                             *
 *                                                                            *
 *  Description : resize window callback routine                              *
 *                                                                            *
 *  Input       : Widget    - caller widget                                   *
 *                XtPointer - client data                                     *
 *                XtPointer - call data                                       *
 *                                                                            *
 *  Ouput       : None                                                        *
 ******************************************************************************/

void 
resize_callback(Widget w, XtPointer client_data, XEvent *event)
{

	Dimension dialog_w;
	Dimension dialog_h;
	Dimension form_w;
	Dimension width;

	DPRINTF(("======================================>\n"));
	if (event->type == ConfigureNotify) {

		XConfigureEvent * cevent = (XConfigureEvent *) event;

		XtVaGetValues(dnd_shell->dnd_dialog,
		    XmNwidth,
		    &dialog_w,
		    XmNheight,
		    &dialog_h,
		    NULL);
		DPRINTF(("resize callback current width:%d height:%d\n",
		    dialog_w, dialog_h));
		DPRINTF(("resize callback cevent width:%d height:%d\n",
		    cevent->width, cevent->height));
		/*
		 * Wait for current height/width to match
		 * event height/width
		 */

		if ((dialog_w != cevent->width) ||
		    (dialog_h != cevent->height)) {
			return;
			/* NOTREACHED */
		}

		XtVaGetValues(dnd_shell->src_frame->src_form,
		    XmNwidth,
		    &form_w,
		    NULL);
		DPRINTF(("- source form width %d\n", form_w));

		/* Update the Source Label */

		width = form_w - src_lbl_delta;
		DPRINTF(("- new src lbl width %d\n", width));
		update_source_label(&file_src_cl, width);


		/* Update the Destination Label */

		width = form_w - dst_lbl_delta;
		DPRINTF(("- new dest lbl width %d\n", width));
		update_destination_label(current_dst_il, width);


		/* Update the information Label */

		width = form_w - info_lbl_delta;
		DPRINTF(("- new info lbl width %d\n", width));
		update_information_label(current_dst_il, width);
	}
}
